# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####
bl_info = {
    "name": "Add Empty Parent",
    "description": "Adds an Empty object as the parent of the active object",    
    "author": "Witold Jaworski",
    "version": (1, 1, 1),
    "blender": (2, 80, 0),
    "location": "Context Menu > Parent > Add Empty Parent",
    "support": "COMMUNITY",
    "category": "Object",
    "warning": "",
    "wiki_url": "",
    "tracker_url": ""
    }
#----- updates
#2017-09-07: this function sets to 0 also the object location (it is transferred to parent).
#2019-08-23: adaptation for Blender 2.8
#----- imports
import bpy
####constants---------------------------------------------------------------
DEBUG = 0 # Debug level. When nonzero, some diagnostics texts are placed at 
        #   output, during processing. Level = 1 generates some genral
        #   statements, Level = 5 - intermediate, while Level = 9 is the most 
        #   detailed. Level > 1 requires PyDev remote Debugger
###--- for direct debugging of this add-on (update the pydevd path!) ---------------------------
if DEBUG > 1:
    import sys
    pydev_path = 'C:/Users/me/.p2/pool/plugins/org.python.pydev.core_7.2.1.201904261721/pysrc'
    if sys.path.count(pydev_path) < 1: sys.path.append(pydev_path)
    import pydevd
    pydevd.settrace(stdoutToServer=True, stderrToServer=True, suspend=False) #stop at first breakpoint
    #Beware: remove all breakpoints from other PyDev projects opened in Eclipse IDE
###-- end remote debug initialization ----------------------------------------------------------

####globals-----------------------------------------------------------------

####classes-----------------------------------------------------------------
####module functions--------------------------------------------------------
def select(context, objects):
    """Helper function: selects specified objects
       Arguments:
            @context (Context):    current context 
            @objects (list):      list of the objects to be selected (the first element will become the active one) 
       This function should be called in the Object Mode, only!
    """    
    bpy.ops.object.select_all(action='DESELECT') #clear current selection
    for obj in objects: #select these objects
        obj.select_set(True)
    if objects: #mark the first object on the list as the active one
        bpy.context.view_layer.objects.active = objects[0]     

def parent_set(context,parent,child):
    """Helper function: sets the parent
       Arguments:
            @context (Context):    current context
            @parent (Object):     the parent object
            @child (Object):      the child object 
       This function should be called in the Object Mode, only!
    """
    select(context, [parent, child])    
    bpy.ops.object.parent_set(type='OBJECT', xmirror=False, keep_transform=True)

def parent_clear(context,child):
    """Helper function: clears the parent
       Arguments:
            @context (Context):    current context
            @child (Object):      a child object 
       This function should be called in the Object Mode, only!
    """
    select(context, [child])    
    bpy.ops.object.parent_clear(type='CLEAR_KEEP_TRANSFORM')
           
#--- ### Blender Operators
class AddEmptyParent(bpy.types.Operator):
    bl_idname = "object.add_empty_parent"
    bl_label = "Add Empty Parent"
    bl_description = "Add an Empty object as the parent of the active object (and as the child of its parent)"
    #--- parameters 
    #--- Blender interface methods
    @classmethod
    def poll(cls,context):
        return (context.mode == 'OBJECT')
       
    def invoke(self, context, event):
        #similar check, like in IntersectMeshes, but different way of communication:
        obj = context.object
        
        if not obj:
            msg = "No active object"
            self.report(type={'ERROR'}, message=msg)
            return {'CANCELLED'}
        else:
            return self.execute(context)
    
    def execute(self, context):
        #bpy.ops.object.editmode_toggle()
        obj = context.object
        parent = obj.parent
        parent_clear(context, obj)
        rotation = list(obj.rotation_euler)
        #create an empty object (on the current layer)
        bpy.ops.object.empty_add(type='ARROWS', align='WORLD', location=obj.location, rotation=(0, 0, 0))
        empty = context.object
        empty.name = obj.name + ".Parent"
        obj.rotation_euler = [0,0,0]
        obj.location = [0,0,0]
        obj.parent = empty
        empty.rotation_euler = rotation
        if parent:
            parent_set(context,parent,empty)
        select(context,[empty])
        return {'FINISHED'}
        
#---------- ### Register -----------
from bpy.utils import register_class, unregister_class

def menu_draw(self, context):
    if (context.object):
        self.layout.operator_context = 'INVOKE_REGION_WIN'
        self.layout.separator()
        self.layout.operator(AddEmptyParent.bl_idname)
       
#list of the classes in this add-on to be registered in Blender API:
classes = [
            AddEmptyParent,
          ]  

def register():
    for cls in classes:
        register_class(cls)
    bpy.types.VIEW3D_MT_object_parent.append(menu_draw)
    if DEBUG: print(__name__ + ": registered")
    
def unregister():
    bpy.types.VIEW3D_MT_object_parent.remove(menu_draw)
    for cls in classes:
        unregister_class(cls)    
    if DEBUG: print(__name__ + ": UNregistered")

    
if __name__ == '__main__':
    register()
